/* The calling syntax is:

            [v] = oradNN(u,setpar)

*/

#include <stdio.h>
#include <math.h>
#include "mex.h"

/* Input Arguments */
#define u_in         prhs[0]
#define setpar_in prhs[1]

/* Output Arguments */
#define v_out        plhs[0]

void mexFunction(nlhs, plhs, nrhs, prhs)
int nlhs, nrhs;
Matrix *plhs[], *prhs[];
{
  double *v, *u, *setpar;

  if (nrhs!=2) mexErrMsgTxt("GRT requires two input arguments.");
  if (nlhs>1) mexErrMsgTxt("GRT returns only one output arguments.");

  if ( mxGetM(u_in) == 0 || mxGetN(u_in) == 0 ) 
    mexErrMsgTxt("Input matrix of size zero..");

  /* Assign pointers to the parameters */
  u      = mxGetPr(u_in);
  setpar = mxGetPr(setpar_in);

  /* Create a matrix for the return argument */
  v_out = mxCreateFull(setpar[3],setpar[5],REAL);

  /* Assign pointers to the output parameters */
  v = mxGetPr(v_out);

  /* Do the actual computations in a subroutine */
  oradon(v,u,setpar);
}

oradon(double *v,double *u,double *setpar)
{
  int m,n, M;
  int r, R;
  int t, T;
  int mmin,mmax,nmin,nmax;
  double sum,x_min,Delta_x,Delta_rho,rho_min,Delta_theta;
  double costheta,sintheta,theta,x,y;
  double dx_isintheta,dx_icostheta,rho,rhooffset;
  double alpha,beta,betap,theta_min;
  double eps=1e-9;
  
  M            = setpar[0];
  x_min        = setpar[1];
  Delta_x      = setpar[2];
  T            = setpar[3];
  Delta_theta  = setpar[4];
  R            = setpar[5];
  rho_min      = setpar[6];
  Delta_rho    = setpar[7];
  theta_min    = setpar[8];
  for(t=0; t<T;t++)
  {
    theta=t*Delta_theta+theta_min;
    sintheta=sin(theta);
    costheta=cos(theta);
    rhooffset=x_min*(costheta+sintheta);
    if (sintheta>sqrt(0.5))
    {
      dx_isintheta=Delta_x/sintheta;
      alpha=-costheta/sintheta;
      for(r=0; r<R; r++ ) 
      {
        beta=(r*Delta_rho+rho_min-rhooffset)/(Delta_x*sintheta);
        betap=beta+0.5;
        if (alpha>eps)
        {
          mmin=(int)ceil(-(betap-eps)/alpha);
          mmax=1+(int)floor((M-betap-eps)/alpha);
        }
        else
          if (alpha<-eps)
          {
            mmin=(int)ceil((M-betap-eps)/alpha);
            mmax=1+(int)floor(-(betap-eps)/alpha);
          }
          else
            if ((betap>0) && (betap<M))
            {
              mmin=0;
              mmax=M;
            }
            else
            {
              mmin=0;
              mmax=-1;
            }
        if (mmin<0) mmin=0;
        if (mmax>M) mmax=M;

        sum=0.0;
        for (m=mmin;m<mmax;m++)
          sum+=u[m+M*(int)(betap+m*alpha)];
        v[t+T*r]=sum*dx_isintheta;
      }
    }
    else
    {
      dx_icostheta=Delta_x/fabs(costheta);
      alpha=-sintheta/costheta;
      for(r=0; r<R; r++ ) 
      {
        beta=(r*Delta_rho+rho_min-rhooffset)/(Delta_x*costheta);
        betap=beta+0.5;
        if (alpha>eps)
        {
          nmin=(int)ceil(-(beta+0.5-eps)/alpha);
          nmax=1+(int)floor((M-0.5-beta-eps)/alpha);
        }
        else 
          if (alpha<-eps)
          {
            nmin=(int)ceil((M-0.5-beta-eps)/alpha);
            nmax=1+(int)floor(-(beta+0.5-eps)/alpha);
          }
          else
            if ((betap>0) && (betap<M))
            {
              nmin=0;
              nmax=M;
            }
            else
            {
              nmin=0;
              nmax=-1;
            }
        if (nmin<0) nmin=0;
        if (nmax>M) nmax=M;

        sum=0.0;
        for (n=nmin;n<nmax;n++)
          sum+=u[(int)(betap+n*alpha)+n*M];
        v[t+T*r]=sum*dx_icostheta;
      }
    }
  }
}
