/* The calling syntax is:

            [v] = oradNN(u,setpar)

*/

#include <stdio.h>
#include <math.h>
#include "mex.h"

/* Input Arguments */
#define u_in         prhs[0]
#define setpar_in prhs[1]

/* Output Arguments */
#define v_out        plhs[0]

void mexFunction(nlhs, plhs, nrhs, prhs)
int nlhs, nrhs;
Matrix *plhs[], *prhs[];
{
  double *v, *u, *setpar;

  if (nrhs!=2) mexErrMsgTxt("GRT requires two input arguments.");
  if (nlhs>1) mexErrMsgTxt("GRT returns only one output arguments.");

  if ( mxGetM(u_in) == 0 || mxGetN(u_in) == 0 ) 
    mexErrMsgTxt("Input matrix of size zero..");

  /* Assign pointers to the parameters */
  u      = mxGetPr(u_in);
  setpar = mxGetPr(setpar_in);

  /* Create a matrix for the return argument */
  v_out = mxCreateFull(setpar[3],setpar[5],REAL);

  /* Assign pointers to the output parameters */
  v = mxGetPr(v_out);

  /* Do the actual computations in a subroutine */
  oradon(v,u,setpar);
}


double sinc(double x)
{

  if (x<0) x=-x;
  if (x<1e-10) 
    return 1.0;
  else
    return sin(x)/x;
}

oradon(double *v,double *u,double *setpar)
{
  int m,n, M;
  int r, R;
  int t, T;
  int mmin,mmax,nmin,nmax,adr;
  double sum,x_min,Delta_x,Delta_rho,rho_min,Delta_theta;
  double costheta,sintheta,theta,x,y;
  double dx_isintheta,dx_icostheta,rho,rhooffset;
  double theta_min;
  double idx,mintrig,psi;
  double *xar,scal;

  M            = setpar[0];
  x_min        = setpar[1];
  Delta_x      = setpar[2];
  T            = setpar[3];
  Delta_theta  = setpar[4];
  R            = setpar[5];
  rho_min      = setpar[6];
  Delta_rho    = setpar[7];
  theta_min    = setpar[8];

  scal=M_PI/Delta_x;
  xar=(double *)malloc(M*sizeof(double));

  for ( m = 0 ; m < M ; m++ ) 
    xar[m]=m*Delta_x+x_min;

  idx=1.0/Delta_x;
  for(t=0; t<T;t++)
  {
    printf("t=%i (%i)\n",t,T);
    theta=t*Delta_theta+theta_min;
    sintheta=sin(theta);
    costheta=cos(theta);
    rhooffset=x_min*(costheta+sintheta);
    if (sintheta>sqrt(0.5))
      mintrig=1.0/sintheta;
    else
      mintrig=1.0/fabs(costheta);

    for(r=0; r<R; r++ )
    {
      rho=r*Delta_rho+rho_min;
      sum=0.0;
      for (m=0;m<M;m++)
      {
        rhooffset=rho-xar[m]*costheta;
        for (n=0;n<M;n++)
        {
          psi=scal*(rhooffset-xar[n]*sintheta);
          sum+=sinc(mintrig*psi)*u[m+n*M];
        }
      }
      v[t+T*r]+=sum*mintrig*Delta_x;
    }
  }
}
