/* The calling syntax is:

   [v] = ipmc(u, setpar)

 */

#include <stdio.h>
#include <math.h>
#include "cmex.h"

/* Input Arguments */
#define u_in       prhs[0]
#define setpar_in  prhs[1]

/* Output Arguments */
#define v_out    plhs[0]


void mexFunction(int nlhs,Matrix * plhs[],int nrhs,Matrix * prhs[])
{
  double *v, *u, *setpar;

  if ( nrhs != 2 )
    mexErrMsgTxt("GRT requires two input arguments.");
  if ( nlhs > 1 )
    mexErrMsgTxt("GRT returns only one output arguments.");

  if ( mxGetM(u_in) == 0 || mxGetN(u_in) == 0 ) 
    mexErrMsgTxt("Input matrix of size zero..");

  /* Assign pointers to the parameters */
  u      = mxGetPr(u_in);
  setpar = mxGetPr(setpar_in);

  /* Create a matrix for the return argument */
  v_out = mxCreateFull(setpar[9],setpar[6],REAL);

  /* Assign pointers to the output parameters */
  v = mxGetPr(v_out);

  /* Do the actual computations in a subroutine */
  hough(v,u,setpar);
}

hough(double* v,double* u,double* setpar)
{
  int n,N;
  int m,M;
  int h,H;
  int index;
  int k,K;
  int k_min,k_max,adr;
  double p,p_min,Delta_p;
  double tau,tau_min,Delta_tau;
  double x,x_min,Delta_x;
  double y,y_min,Delta_y,hfloat;
  double d,alpha,beta,yoff,eps=1e-9,reldtau;

  M=setpar[0];
  x_min=setpar[1];
  Delta_x=setpar[2];
  N=setpar[3];
  y_min=setpar[4];
  Delta_y=setpar[5];
  H=setpar[6];
  tau_min=setpar[7];
  Delta_tau=setpar[8];
  K=setpar[9];
  p_min=setpar[10];
  Delta_p=setpar[11];
  
  
  for(index=0;index<K*H;index++)
    v[index]=0.0;
  
  for(m=0;m<M;m++)
  {
    x=x_min+m*Delta_x;
    yoff=tau_min+x*p_min;
    for(n=0;n<N;n++)
    {
      d=u[m+n*M];
      if(fabs(d)>eps)
      {
        alpha=-x*Delta_p/Delta_tau;
        beta=(y_min+n*Delta_y-yoff)/Delta_tau;
      
        if (alpha>eps)
        {
          k_min=(int)ceil((eps-beta)/alpha);
          k_max=1+(int)floor((H-1-beta-eps)/alpha);
        }
        else
          if (alpha<-eps)
          {
            k_min=(int)ceil((H-1-beta-eps)/alpha);
            k_max=1+(int)floor((eps-beta)/alpha);
          }
          else
          {
            k_min=0;
            k_max=K;
          }
        if (k_min<0) k_min=0;
        if (k_max>K) k_max=K;
        for(k=k_min;k<k_max;k++)
        {
          hfloat=alpha*k+beta;
          h=(int)hfloat;
          reldtau=(hfloat-h)/Delta_tau;
          adr=k+h*K;
          v[adr]+=d*(1-reldtau);
          v[adr+K]+=d*reldtau;
        }
      }
    }
  }
}
