/* The calling syntax is:

            [v] = ipmc(u, setpar)

*/

#include <stdio.h>
#include <math.h>
#include "cmex.h"

/* Input Arguments */
#define u_in       prhs[0]
#define setpar_in  prhs[1]

/* Output Arguments */
#define v_out    plhs[0]

void mexFunction(nlhs, plhs, nrhs, prhs)
int nlhs, nrhs;
Matrix *plhs[], *prhs[];
{
  double *v, *u, *setpar;

  if ( nrhs != 2 )
    mexErrMsgTxt("IPM requires two input arguments.");
  if ( nlhs > 1 )
    mexErrMsgTxt("IPM returns only one output arguments.");

  if ( mxGetM(u_in) == 0 || mxGetN(u_in) == 0 ) 
    mexErrMsgTxt("Input matrix of size zero..");

  /* Assign pointers to the input parameters */
  u      = mxGetPr(u_in);
  setpar = mxGetPr(setpar_in);

  /* Create a matrix for the return argument */
  v_out = mxCreateFull(setpar[9], setpar[6], REAL);

  /* Assign pointers to the output parameters */
  v = mxGetPr(v_out);

  /* Do the actual computations in a subroutine */
  ipmc(v,u,setpar);
}



ipmc(v,u,setpar)
double *v, *u, *setpar;
{
  int n, N;
  int m, M;
  int h, H;
  int index;
  int k, K;
  int h_min,h_max;
  double p,p_min,Delta_p;
  double tau,tau_min,Delta_tau;
  double x,x_min,Delta_x;
  double y,y_min,Delta_y;
  double d;

  M          = setpar[0];
  x_min      = setpar[1];   
  Delta_x    = setpar[2]; 
  N          = setpar[3];    
  y_min      = setpar[4];  
  Delta_y    = setpar[5];
  H          = setpar[6];
  tau_min    = setpar[7];
  Delta_tau  = setpar[8];
  K          = setpar[9];
  p_min      = setpar[10];
  Delta_p    = setpar[11];


  for ( index = 0 ; index < K*H ; index++ )
      v[index] = 0.0;

  for ( m = 0 ; m < M ; m++ )
  {
    x=x_min+m*Delta_x;
    for ( n = 0 ; n < N ; n++ )
    {
      y=y_min+n*Delta_y;

      d=u[m+n*M];
      if (d!=0)
      {
        for ( k = 0 ; k < K ; k++ )
        {
          p=p_min+k*Delta_p;
          tau=y-p*x;
          h=(int)floor((tau-tau_min)/Delta_tau+0.5);
          if ((h>=0) && (h<H))
            v[k+h*K] += d;
        }
      }
    }
  }
}
