/* The calling syntax is:

   [v] = ipmc(u, setpar)

 */

#include <stdio.h>
#include <math.h>
#include "cmex.h"

/* Input Arguments */
#define u_in       prhs[0]
#define setpar_in  prhs[1]

/* Output Arguments */
#define v_out    plhs[0]


void mexFunction(int nlhs,Matrix * plhs[],int nrhs,Matrix * prhs[])
{
  double *v, *u, *setpar;

  if ( nrhs != 2 )
    mexErrMsgTxt("GRT requires two input arguments.");
  if ( nlhs > 1 )
    mexErrMsgTxt("GRT returns only one output arguments.");

  if ( mxGetM(u_in) == 0 || mxGetN(u_in) == 0 ) 
    mexErrMsgTxt("Input matrix of size zero..");

  /* Assign pointers to the parameters */
  u      = mxGetPr(u_in);
  setpar = mxGetPr(setpar_in);

  /* Create a matrix for the return argument */
  v_out = mxCreateFull(setpar[9],setpar[6],REAL);

  /* Assign pointers to the output parameters */
  v = mxGetPr(v_out);

  /* Do the actual computations in a subroutine */
  hough(v,u,setpar);
}

hough(double* v,double* u,double* setpar)
{
  int n,N;
  int m,M;
  int h,H;
  int index;
  int k,K;
  int k_min,k_max;
  double p,p_min,Delta_p;
  double tau,tau_min,Delta_tau;
  double x,x_min,Delta_x;
  double y,y_min,Delta_y;
  double d,alpha,beta,betap,yoff,eps=1e-9;

  M=setpar[0];
  x_min=setpar[1];
  Delta_x=setpar[2];
  N=setpar[3];
  y_min=setpar[4];
  Delta_y=setpar[5];
  H=setpar[6];
  tau_min=setpar[7];
  Delta_tau=setpar[8];
  K=setpar[9];
  p_min=setpar[10];
  Delta_p=setpar[11];
  
  mexPrintf("X=%i  Y=%i  P=%i  TAU=%i\n",M,N,K,H);
  mexPrintf("x_min=%f  y_min=%f  p_min=%f  tau_min=%f\n",
            x_min,y_min,p_min,tau_min);
  mexPrintf("Dx=%f  Dy=%f  Dp=%f  Dtau=%f\n",
            Delta_x,Delta_y,Delta_p,Delta_tau);
  
  for(index=0;index<K*H;index++)
    v[index]=0.0;
  
  for(m=0;m<M;m++)
  {
    x=x_min+m*Delta_x;
    yoff=tau_min+x*p_min;
    for(n=0;n<N;n++)
    {
      d=u[m+n*M];
      if(fabs(d)>eps)
      {
        alpha=-x*Delta_p/Delta_tau;
        beta=(y_min+n*Delta_y-yoff)/Delta_tau;
        betap=beta+0.5;
      
        if (alpha>eps)
        {
          k_min=(int)ceil((eps-betap)/alpha);
          k_max=1+(int)floor((H-betap-eps)/alpha);
        }
        else
          if (alpha<-eps)
          {
            k_min=(int)ceil((H-betap-eps)/alpha);
            k_max=1+(int)floor((eps-betap)/alpha);
          }
          else
            if ((beta>1) & (beta<H-1))
            {
              k_min=0;
              k_max=K;
            }
            else
            {
              k_min=0;
              k_max=-1;
            }
        if (k_min<0) k_min=0;
        if (k_max>K) k_max=K;
        for(k=k_min;k<k_max;k++)
        {
          if (((int)(alpha*k+betap)<0) || 
              (int)(alpha*k+betap)>=H)
            printf("er k=%i h=%i m=%i n=%i alpha=%f beta=%f\n",
                   k,(int)(alpha*k+betap),m,n,alpha,beta);
          v[k+K*(int)(alpha*k+betap)]+=d;
        }
      }
    }
  }
}
