/* The calling syntax is:

            [v] = radc(u,setpar)

*/

#include <stdio.h>
#include <math.h>
#include "mex.h"

/* Input Arguments */
#define u_in       prhs[0]
#define setpar_in  prhs[1]

/* Output Arguments */
#define v_out      plhs[0]

void mexFunction(nlhs, plhs, nrhs, prhs)
int nlhs, nrhs;
Matrix *plhs[], *prhs[];
{
  double *v, *u, *setpar;

  if ( nrhs != 2 )
    mexErrMsgTxt("GRT requires two input arguments.");
  if ( nlhs > 1 )
    mexErrMsgTxt("GRT returns only one output arguments.");

  if ( mxGetM(u_in) == 0 || mxGetN(u_in) == 0 ) 
    mexErrMsgTxt("Input matrix of size zero..");

  /* Assign pointers to the parameters */
  u      = mxGetPr(u_in);
  setpar = mxGetPr(setpar_in);

  /* Create a matrix for the return argument */
  v_out = mxCreateFull(setpar[9],setpar[6],REAL);

  /* Assign pointers to the output parameters */
  v = mxGetPr(v_out);

  /* Do the actual computations in a subroutine */
  radon(v,u,setpar);
}

radon(v,u,setpar)
double *v, *u, *setpar;
{
  int yint, YSamples,YSamplesm1;
  int xint, XSamples;
  int pint, TauSamples;
  int tauint, PSamples;
  int indexp;
  double dy,rsum,p,plow,DeltaP,x,xlow,DeltaX;
  double y,ylow,DeltaY,tau,taulow,DeltaTau,invDeltaY;
  double *tauar,*xar;

  XSamples   = setpar[0];
  xlow       = setpar[1];   
  DeltaX     = setpar[2]; 
  YSamples   = setpar[3];    
  ylow       = setpar[4];  
  DeltaY     = setpar[5];
  TauSamples = setpar[6];
  taulow     = setpar[7];
  DeltaTau   = setpar[8];
  PSamples   = setpar[9];
  plow       = setpar[10];
  DeltaP     = setpar[11];

  invDeltaY=1.0/DeltaY;
  YSamplesm1=YSamples-1;

  mexPrintf("X=%i  Y=%i  P=%i  TAU=%i\n",
            XSamples,YSamples,PSamples,TauSamples);
  mexPrintf("xlow=%f  ylow=%f  plow=%f  taulow=%f\n",
            xlow,ylow,plow,taulow);
  mexPrintf("Dx=%f  Dy=%f  Dp=%f  Dtau=%f\n",
            DeltaX,DeltaY,DeltaP,DeltaTau);
             
  tauar=(double *)malloc(TauSamples*sizeof(double));
  xar=(double *)malloc(XSamples*sizeof(double));

  for ( tauint = 0 ; tauint < TauSamples ; tauint++ ) 
    tauar[tauint]=tauint*DeltaTau+taulow;

  for ( xint = 0 ; xint < XSamples ; xint++ ) 
    xar[xint]=xint*DeltaX+xlow;

  for ( pint = 0 ; pint < PSamples ; pint++ )
  {
    p=plow+DeltaP*pint;
    for ( tauint = 0 ; tauint < TauSamples ; tauint++ )
    {
      tau=tauar[tauint];
      rsum=0.0;
      for ( xint = 0 ; xint < XSamples ; xint++ )
      {
        y=(p*xar[xint]+tau-ylow)*invDeltaY;
        yint=(int)floor(y);
        dy=(y-yint)*invDeltaY;
        if ((yint>=0) && (yint<YSamplesm1)) 
          rsum+=(1-dy)*u[xint+yint*XSamples]+dy*u[xint+(yint+1)*XSamples];
      }
      v[tauint*PSamples+pint]=rsum;
    }
  }
  free(tauar);
  free(xar);
}
