/* The calling syntax is:

            [v] = radc(u,setpar)

*/

#include <stdio.h>
#include <math.h>
#include "mex.h"

/* Input Arguments */
#define u_in       prhs[0]
#define setpar_in  prhs[1]

/* Output Arguments */
#define v_out      plhs[0]

void mexFunction(int nlhs,Matrix * plhs[],int nrhs,Matrix * prhs[])
{
  double *v, *u, *setpar;

  if ( nrhs != 2 )
    mexErrMsgTxt("GRT requires two input arguments.");
  if ( nlhs > 1 )
    mexErrMsgTxt("GRT returns only one output arguments.");

  if ( mxGetM(u_in) == 0 || mxGetN(u_in) == 0 ) 
    mexErrMsgTxt("Input matrix of size zero..");

  /* Assign pointers to the parameters */
  u      = mxGetPr(u_in);
  setpar = mxGetPr(setpar_in);

  /* Create a matrix for the return argument */
  v_out = mxCreateFull(setpar[9],setpar[6],REAL);

  /* Assign pointers to the output parameters */
  v = mxGetPr(v_out);

  /* Do the actual computations in a subroutine */
  radon(v,u,setpar);
}

radon(double* v,double* u,double* setpar)
{
  int n, N;
  int m, M;
  int k, H;
  int h, K;
  int indexp,mmin,mmax;
  double rsum,p,p_min,Delta_p,x,x_min,Delta_x;
  double y,y_min,Delta_y,tau,tau_min,Delta_tau;
  double *tauar;
  double nfloat,alpha,beta,invDelta_y,eps=1e-8,betap,tauoff;
  int nerr;

  M           = setpar[0];
  x_min       = setpar[1];   
  Delta_x     = setpar[2]; 
  N           = setpar[3];    
  y_min       = setpar[4];  
  Delta_y     = setpar[5];
  H           = setpar[6];
  tau_min     = setpar[7];
  Delta_tau   = setpar[8];
  K           = setpar[9];
  p_min       = setpar[10];
  Delta_p     = setpar[11];

  invDelta_y=1.0/Delta_y;

  mexPrintf("X=%i  Y=%i  P=%i  TAU=%i\n",M,N,K,H);
  mexPrintf("x_min=%f  y_min=%f  p_min=%f  tau_min=%f\n",
            x_min,y_min,p_min,tau_min);
  mexPrintf("Dx=%f  Dy=%f  Dp=%f  Dtau=%f\n",
            Delta_x,Delta_y,Delta_p,Delta_tau);
             
  tauar=(double *)malloc(H*sizeof(double));

  for (h=0;h<H;h++) 
    tauar[h]=h*Delta_tau+tau_min;

  for (k=0;k<K;k++)
  {
    p=p_min+Delta_p*k;
    alpha=p*Delta_x*invDelta_y;
    tauoff=p*x_min-y_min;
    for (h=0;h<H;h++)
    {
      beta=(tauoff+tauar[h])*invDelta_y;
      betap=beta+0.5;
      if (alpha>eps)
      {
        mmin=(int)ceil((eps-betap)/alpha);
        mmax=1+(int)floor((N-betap-eps)/alpha);
      }
      else
        if (alpha<-eps)
        {
          mmin=(int)ceil((N-betap-eps)/alpha);
          mmax=1+(int)floor(-(betap-eps)/alpha);
        }
        else
          if ((beta>1) & (beta<H-1))
          {
            mmin=0;
            mmax=M;
          }
          else
          {
            mmin=0;
            mmax=-1;
          }
      if (mmin<0) mmin=0;
      if (mmax>M) mmax=M;
      nfloat=betap+mmin*alpha;
      rsum=0.0;
      for (m=mmin;m<mmax;m++) rsum+=u[m+M*(int)(betap+m*alpha)];
      v[k+h*K]=rsum*Delta_x;
    }
  }
  free(tauar);
}
